searchData={"content_type":"text/markdown","items":[{"doc":"Stoker makes sure that your cluster keeps running, no matter\nwhat happens.\n\nOne of the big ideas behind Elixir/Erlang is distribution as\na way to address faults - if you have multiple servers running\nin a cluster, and one of them dies, the others keep on churning.\nThis is quite easy to do when those servers are all alike, e.g.\na webserver running a Phoenix app - wherever the request lands,\nit is processed there. But you can do this easily is any environment -\nJava, Go, whatever.\n\nWhat is more interesting in the Elixir/Erlang\nworld is the ability to have **cluster-unique processes** that end up\nbeing distribuited and surviving machine faults. This happens\na lot of times to me - I often have to write data import jobs\nthet retrieve, rewrite and forward data. I want them to be\nalways available, will accept small glitches (because a process may\ndie on my end or on the other end) but I never want two processes\nto run the same job at the same time.\n\nStoker is the foundation upon which this is built. A single\ninstance of Stoker is always running in the cluster - and it it\ndies, another one is restarted on a surviving node. It has\nfull visibility of the cluster, and a list of jobs to assure being up.\nIt will try to make sure that all of them are available, and\nif some are missing, will restart them on one of the available nodes.\nUsually, this happens by delegating them to a local DynamicSupervisor,\nthat once started will do its magic to keep the process running.\n\nEvery once in a while, or when receiving a wake-up signal, it\nwill wake up again and make sure that\neverything is in order.  This is beacuse it is quite likely that\nyou have a dynamic list of jobs you want to run - e.g. you\nmay add or remove them for a database table - so Stoker will\nreact automatically when you add a new one. It is also possible\nthat some external conditions had the DynamicSupervisor give up,\nso it may be appropriate to restart the process on another random\nnode and try again.","ref":"Stoker.html","title":"Stoker","type":"behaviour"},{"doc":"Here is a typical scenario running Stoker. Our application\nruns on two nodes, and our goal is to keep Process X alive\nsomewhere.\n\n \n\n       Node 1      Node 2        Application        Dyn.Sup.        Stoker.Activator      Leader        Your module      Custom logic implementing behavior @Stoker        Application        Stoker.Activator      Follower        Dyn.Sup.        Process X                             monitors    \n\n \n\n\n\nOn start-up, the Stoker Activator process on node 1 becomes active (we say it's\na Leader),\nand the one on node 2, that was started a few seconds after that,\nnoticed that there was already\na running one, and became a Follower of the Leader,\nwaiting for its brother on node 1 to terminate.\n\nSo the process on node 1 called into your module - one you\ncreated implementing the\nStoker behavior - and asked it to check and\nactivate all processes it needed to. Your module choose a random\nDynamicSupervisor out of the node pool, selected the one on node 2,\n and asked it to\nstart supervising Process X.\n\nEvery once in a while, Stoker on node 1 wakes up and calls your module\nto make sure that all processes that are supposed to be alive actually are;\nif not, they are started again.\n\n#","ref":"Stoker.html#module-implementation","title":"Implementation - Stoker","type":"behaviour"},{"doc":"Stoker on node 2 will notice that its brother failed, so it will\nbecome a leader. It will then trigger your module, that will notice that\nProcess X is available, and do nothing more. If there were some\npersistent processes on node 1, they would be started again on node 2,\nas that is the only remaining node.\n\nIf node 1 restarts, Stoker on node 1 will notice that there is already\na Leader on node 2, so it will become a Follower - it will\nput a watch on the Leader and will wait for\nit to become unavailable.\n\n#","ref":"Stoker.html#module-what-happens-if-node-1-dies","title":"What happens if node 1 dies? - Stoker","type":"behaviour"},{"doc":"If node 2 dies, Stoker on node 1 - that is already a Leader -\nreceives an update that the\ncluster composition changed; it will trigger your module, that will\nnotice that Process X is not running, and will start it again\non the only remaining node.\n\nWhen node 2 restarts, Stoker on it will notice that there is already\na Leader on node 1, and will become a Follower.\n\n\n#","ref":"Stoker.html#module-what-happens-if-node-2-dies","title":"What happens if node 2 dies? - Stoker","type":"behaviour"},{"doc":"If we add a new node, Stoker on node 1 receives an update that the\ncluster composition changed; it will trigger your module, that will\nnotice that Process X is still running, and won't do anything.\n\nThe new Stoker on node 3 will become a Follower, like the one on\nnode 2 is.\nIf node 1 becomes unavailable, both will race to become the new\nLeader; one of them will succeed, and the other one will become a\nFollower.\n\nAs node 3 has its own DynamicSupervisor, it will become eligible\nto run new processes as the need arises.\n\n#","ref":"Stoker.html#module-what-happens-if-we-add-node-3","title":"What happens if we add node 3? - Stoker","type":"behaviour"},{"doc":"This is the tough one.\n\nOn a netsplit, both nodes keep running but none of them sees\nthe other one. In this case, each Stoker will become Leader\nand run Process X on its own pool. In any case, your module will\nbe notified, so you could decide what to do - terminate all processes,\nwait a bit, whatever.\n\nWhen the netsplit heals, one of the Stoker processes is terminated,\nand the same happens for every processed that has registered twice.\nSo one of the processes will remain leader, the one that was terminated\nwill respawn and become a follower.\n\n#","ref":"Stoker.html#module-network-partitions","title":"Network partitions - Stoker","type":"behaviour"},{"doc":"There is no facility to do rebalancing yet, but as your module is\ntriggered on network events, you could do that.","ref":"Stoker.html#module-rebalancing","title":"Rebalancing - Stoker","type":"behaviour"},{"doc":"To guarantee uniqueness of running processes, we use the battle-tested\n`:global` naming module, that will make sure that a name is registered\nonly once.\n\nIt's not very fast, but can manage thousands of registrations per\nseconds on moderately-sized clusters and it's supposedly very\nreliable, so it's good enough for what\nwe need here.\n\n# Using in practice","ref":"Stoker.html#module-guaranteeing-uniqueness","title":"Guaranteeing uniqueness - Stoker","type":"behaviour"},{"doc":"The Activator is implemented by `Stoker.Activator`,\nthat is a GenServer that will call your Stoker behavior\nwhen needed.\n\nSo in your Application sequence you will need\nto add:\n\n````\n{DynamicSupervisor, [name: {StokerDS, node()}, strategy: :one_for_one]},\n{Stoker.Activator, xx.MyStoker},\n````\n\nIn the first row, we ask each node of the cluster\nto start up and register on `:global` a `DynamicSupervisor`\nthat is called `{StokerDS, node1@cluster}`. This way\nwe can address it easily from any node in the cluster.\n\nOn the second row, we start a `Stoker.Activator`  GenServer\nthat, when acting as a leader, registers\nitself as `{Stoker.Activator, your_module_name}`,\nso you can definitely have more than one running on the\nsame cluster.","ref":"Stoker.html#module-start-up-within-an-application","title":"Start-up within an application - Stoker","type":"behaviour"},{"doc":"The life-cycle of a Stoker call-back is modelled on\nthe one that a `GenServer` offers.\n\nThere is a state term that can be used to store a\nstate to be held between multiple calls (but only on\nthe same server - the state is not shared with followers).\n\n\n \n\n         leader      follower      splitbrain            now_leader      now_follower      now_leader      cluster_change, timer, trigger      cluster_split      cluster_change      shutdown      shutdown      shutdown","ref":"Stoker.html#module-the-stoker-life-cycle","title":"The Stoker life-cycle - Stoker","type":"behaviour"},{"doc":"In front of a cluster change, determines whether\nwe are on the losing side of a netsplit situation or not.\n\nFor example, if we have a three node cluster, we may protect\nagainst netsplits by saying that any node left that has\nless than two nodes is on the losing side of a netsplit,\nso should terminate all processes.\n\nIf the answer is:\n\n- `:yes` - calls event `:cluster_change`\n- `:cluster_split` - the cluster is invalid, that is\n  we are on the losing side of a net-split, so call\n  event `:cluster_split` so local processes can be terminated\n- `no` - the cluster is invalid, but do not raise any event","ref":"Stoker.html#c:cluster_valid?/1","title":"Stoker.cluster_valid?/1","type":"callback"},{"doc":"","ref":"Stoker.html#c:event/3","title":"Stoker.event/3","type":"callback"},{"doc":"Hello world.","ref":"Stoker.html#hello/0","title":"Stoker.hello/0","type":"function"},{"doc":"iex> Stoker.hello()\n    :world","ref":"Stoker.html#hello/0-examples","title":"Examples - Stoker.hello/0","type":"function"},{"doc":"","ref":"Stoker.html#c:init/0","title":"Stoker.init/0","type":"callback"},{"doc":"","ref":"Stoker.html#c:next_timer_in/1","title":"Stoker.next_timer_in/1","type":"callback"},{"doc":"","ref":"Stoker.html#t:activator_event/0","title":"Stoker.activator_event/0","type":"type"},{"doc":"","ref":"Stoker.html#t:activator_state/0","title":"Stoker.activator_state/0","type":"type"},{"doc":"This GenServer is used to make sure that a process will\nalways - and once - be active in the cluster.\n\nThe way this works is that we use the  `:global` naming\nservice as a mutex - if the name {Activator, YourModule} can be registered,\nthen this server runs the \"correct\" activator.\n\nIf it cannot be registered, this means somebody else is holding it\n- so we put a monitor on the process who actually registered the name,\nand when it dies, we try registering instead.\n\nOn every application, its Activator (or Activators) are always\nstarted once per server, so they \"idle\" ones will just be waiting\nfor their turn to take over. If a new server joins a healthy cluster,\nits Activator will simply be waiting.\n\nOnce achieved, the only way to lose one's :leader status is by\ndeath, or disconnection from the main cluster.\n\nTODO: Decommissioning and cluster limits","ref":"Stoker.Activator.html","title":"Stoker.Activator","type":"module"},{"doc":"A module implementing the Stoker callback will be called\nwhen some of these events happen:\n\n- A GenServer becomes master\n- A node joins or leaves the cluster\n- Every few minutes","ref":"Stoker.Activator.html#module-the-stoker-callback","title":"The Stoker callback - Stoker.Activator","type":"module"},{"doc":"This GenServer has a state that can be queried by using the\n`dump/1` system call.\n\nIt contains:\n\n- `module`: the Stoker module to be called back. This is the\nsame module that the Activator is registered under.\n- `mod_state`: a provate state for the Stoker module. This state is\n per-machine - this means that when another node becomes :leader,\n- `state`:  :unk,\n- `created_at`: When this GenServer was started (even if not :leader)\n- `active_from`:  When this GenServer became :leader, or nil\n- `leader_pid`:  The current PID, when :leader\n- `monitor_leader`:  The monitor reference to the current :leader, if not :leader\n- `current_node`: The name of the current node.","ref":"Stoker.Activator.html#module-the-state","title":"The state - Stoker.Activator","type":"module"},{"doc":"","ref":"Stoker.Activator.html#build_timer/1","title":"Stoker.Activator.build_timer/1","type":"function"},{"doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","ref":"Stoker.Activator.html#child_spec/1","title":"Stoker.Activator.child_spec/1","type":"function"},{"doc":"","ref":"Stoker.Activator.html#dump/1","title":"Stoker.Activator.dump/1","type":"function"},{"doc":"","ref":"Stoker.Activator.html#event/3","title":"Stoker.Activator.event/3","type":"function"},{"doc":"","ref":"Stoker.Activator.html#start_link/1","title":"Stoker.Activator.start_link/1","type":"function"},{"doc":"","ref":"Stoker.Activator.html#whereis/1","title":"Stoker.Activator.whereis/1","type":"function"},{"doc":"","ref":"Stoker.Activator.html#t:level/0","title":"Stoker.Activator.level/0","type":"type"},{"doc":"Functions to handle clusters.","ref":"Stoker.Cluster.html","title":"Stoker.Cluster","type":"module"},{"doc":"Show all `:global` names and their PIDs.","ref":"Stoker.Cluster.html#ps/0","title":"Stoker.Cluster.ps/0","type":"function"}]}